#!/bin/bash
# Copyright 2010 - 2019 Michal Žejdl
# called from dispatcher.sh
# run desired subcollectors one by one

confdir=$(dirname $0)/../../core
config=$confdir/defaults.sh
test -f $config || exit
source $config

agentid=$1
version=$2
address=$3
sysObjectID=$4
ifNumber=$5
dot1dStp=$6
dot1dTpFdbEntry=$7
hostResources=$8
ifXEntry=$9
entity=${10}

mibdir=$coredir/snmp/mib
get_succeed=n

if test $version -eq 6; then
	snmpaddress=udp6:[$address]
else
	snmpaddress=$address
fi

if test "${sysObjectID:0:13}" = ".1.3.6.1.4.1."; then
	# valid prefix
	cut=${sysObjectID#.1.3.6.1.4.1.}
	organization=${cut%%.*}
	test -z "$organization" && organization=unknown
else
	organization=unknown
fi

#echo "$(date '+%F %T') $(basename $0) sysObjectID $sysObjectID organization $organization"

# a new file descriptor to prevent pipe from closing
exec 3> $mysqlwpipe

#echo "$(date '+%F %T') $(basename $0) agent $agentid address $address gs $get_succeed"

case "$organization" in
9)
	$mibdir/cisco.sh "$snmpaddress" "$agentid" \
		&& get_succeed=y
	;;
43)
	# do not try to collect cpu/mem from old switches
	test ${sysObjectID:0:28} != ".1.3.6.1.4.1.43.10.27.4.1.2." \
			-a ${sysObjectID:0:20} != ".1.3.6.1.4.1.43.1.8." \
		&& $mibdir/3com.sh "$snmpaddress" "$agentid" \
		&& get_succeed=y
	;;
318)
	$mibdir/apc.sh "$snmpaddress" "$agentid" \
		&& get_succeed=y
	;;
776)
	$mibdir/telindus.sh "$snmpaddress" "$agentid" \
		&& get_succeed=y
	;;
12140)
	$mibdir/alcoma.sh "$snmpaddress" "$agentid" \
		&& get_succeed=y
	;;
14988)
	$mibdir/mikrotik.sh "$snmpaddress" "$agentid" \
		&& get_succeed=y
	;;
esac

#echo "$(date '+%F %T') $(basename $0) agent $agentid address $address gs $get_succeed"

test $ifNumber -gt 0 && \
	$mibdir/if.sh "$snmpaddress" "$agentid" "$organization" "$ifXEntry" && \
		get_succeed=y

#echo "$(date '+%F %T') $(basename $0) agent $agentid address $address gs $get_succeed"

test "$dot1dStp" = "y" \
	     && "$mibdir"/dot1dStp.sh "$snmpaddress" "$agentid" \
	     && get_succeed=y

test "$dot1dTpFdbEntry" = "y" \
	     && "$mibdir"/dot1d.sh "$snmpaddress" "$agentid" "$organization" \
	     "$sysObjectID" \
	     && get_succeed=y

test "$hostResources" = "y" && \
	$mibdir/hostResources.sh "$snmpaddress" "$agentid" \
		&& get_succeed=y

test "$entity" = "y" && \
	$mibdir/entity.sh "$snmpaddress" "$agentid" \
		&& get_succeed=y

#echo "$(date '+%F %T') $(basename $0) agent $agentid address $address gs $get_succeed"

test $get_succeed = y && echo "update snmp$version set get = unix_timestamp() where address = '$address';" >&3

# unix_timestamp() shifts next collecting by overhead time
# this leads to better distribution of collecting in $snmpinterval
echo "update snmpagent set collecting = 'n', collected = unix_timestamp() where id = '$agentid';" >&3
