#!/bin/bash
# Copyright 2010 - 2018 Michal Žejdl
# fetch some enterprises.318 OIDs and store to mysql table mibapc or RRD

confdir=$(dirname $0)/../..
config=$confdir/defaults.sh
test -f $config || exit
. $config

functions=$coredir/snmp/functions.sh
test -f $functions || exit
. $functions

address=$1
agent=$2

errlog=$logdir/snmp/$(basename $0 .sh).err

# a new file descriptor to prevent pipe from closing
exec 3>$mysqlwpipe

# group write permissions for rrdcached
umask 002

rrdstep=$snmpinterval
rrdheartbeat=$((2 * rrdstep))
rraxff=0.5
rrathirtym=$((30 * 60 / rrdstep))
rratwoh=$((4 * rrathirtym))
rraday=$((12 * rratwoh))
rraweek=$((7 * rraday))
rrarows=600

rrddestdir=$rrddir/snmpagent/$agent/APC
test -d "$rrddestdir" || mkdir -p "$rrddestdir" || exit

oldifs=$IFS
# one line = one array element; and focus only on the first line
IFS=$'\n'
# -Oa to get rid of Hex-STRING triggered by some chars
# some UPSes sometimes don't answer to this get,
#	and then -r 0 approach to speed things up leads to discontinuous graphs
result=($(snmpget -m '' -v 1 -c $snmprcommunity -Oanqt $address \
.1.3.6.1.4.1.318.1.1.1.1.1.1.0 \
.1.3.6.1.4.1.318.1.1.1.2.1.2.0 \
.1.3.6.1.4.1.318.1.1.1.2.2.1.0 \
.1.3.6.1.4.1.318.1.1.1.2.2.2.0 \
.1.3.6.1.4.1.318.1.1.1.2.2.3.0 \
.1.3.6.1.4.1.318.1.1.1.3.2.1.0 \
.1.3.6.1.4.1.318.1.1.1.4.2.3.0 \
.1.3.6.1.4.1.318.1.1.1.7.2.3.0 \
.1.3.6.1.4.1.318.1.1.1.7.2.4.0 \
.1.3.6.1.4.1.318.1.1.1.7.2.6.0 \
.1.3.6.1.4.1.318.1.1.1.7.2.7.0 \
.1.3.6.1.4.1.318.1.1.1.11.1.1.0 \
))

snmp_status=$?

# by my observation status 1 means no returned values at all so leave now
#	and don't bother with parsing $result
if test $snmp_status -eq 1
then
	echo "$(date '+%F %T') $(basename $0) address $address agent $agent snmp_status $snmp_status" >>$errlog
	exit $snmp_status
fi

IFS=$oldifs

# prepare for inserting into myslq after incomplete response
upsBasicIdentModel=null
upsBasicBatteryTimeOnBattery=null
upsAdvBatteryCapacity=null
upsAdvBatteryTemperature=null
upsAdvBatteryRunTimeRemaining=null
upsAdvOutputLoad=null
upsAdvTestDiagnosticsResults=null
upsAdvTestLastDiagnosticsDate=null
upsAdvTestCalibrationResults=null
upsAdvTestCalibrationDate=null
upsBasicStateOutputState=null
# the same for RRD
upsAdvInputLineVoltage=U

values=0

for ((oid_row = 0; oid_row < ${#result[@]}; oid_row++)); do

	test "${result[oid_row]::22}" = ".1.3.6.1.4.1.318.1.1.1" && ((values++))

	case ${result[oid_row]%% *} in
	.1.3.6.1.4.1.318.1.1.1.1.1.1.0)
		# without MIB (-m '') there are quotes around strings
		upsBasicIdentModel=${result[$oid_row]#* \"}
		upsBasicIdentModel=${upsBasicIdentModel%\"}
		replace_specchars "$upsBasicIdentModel" \
			&& upsBasicIdentModel=$without_specchars
		;;
	.1.3.6.1.4.1.318.1.1.1.2.1.2.0)
		upsBasicBatteryTimeOnBattery=${result[$oid_row]#* }
		is_natural_number "$upsBasicBatteryTimeOnBattery" \
			|| upsBasicBatteryTimeOnBattery=null
		;;
	.1.3.6.1.4.1.318.1.1.1.2.2.1.0)
		upsAdvBatteryCapacity=${result[$oid_row]#* }
		is_natural_number "$upsAdvBatteryCapacity" \
			|| upsAdvBatteryCapacity=null
		;;
	.1.3.6.1.4.1.318.1.1.1.2.2.2.0)
		upsAdvBatteryTemperature=${result[$oid_row]#* }
		is_natural_number "$upsAdvBatteryTemperature" \
			|| upsAdvBatteryTemperature=null
		;;
	.1.3.6.1.4.1.318.1.1.1.2.2.3.0)
		upsAdvBatteryRunTimeRemaining=${result[$oid_row]#* }
		is_natural_number "$upsAdvBatteryRunTimeRemaining" \
			|| upsAdvBatteryRunTimeRemaining=null
		;;
	.1.3.6.1.4.1.318.1.1.1.3.2.1.0)
		upsAdvInputLineVoltage=${result[$oid_row]#* }
		is_natural_number "$upsAdvInputLineVoltage" \
			|| upsAdvInputLineVoltage=U
		;;
	.1.3.6.1.4.1.318.1.1.1.4.2.3.0)
		upsAdvOutputLoad=${result[$oid_row]#* }
		is_natural_number "$upsAdvOutputLoad" \
			|| upsAdvOutputLoad=null
		;;
	.1.3.6.1.4.1.318.1.1.1.7.2.3.0)
		upsAdvTestDiagnosticsResults=${result[$oid_row]#* }
		# int, but defined values are 1 - 4
		is_natural_number "$upsAdvTestDiagnosticsResults" \
			|| upsAdvTestDiagnosticsResults=null
		;;
	.1.3.6.1.4.1.318.1.1.1.7.2.4.0)
		upsAdvTestLastDiagnosticsDate=${result[$oid_row]#* \"}
		upsAdvTestLastDiagnosticsDate=${upsAdvTestLastDiagnosticsDate%\"}
		replace_specchars "$upsAdvTestLastDiagnosticsDate" \
			&& upsAdvTestLastDiagnosticsDate=$without_specchars
		;;
	.1.3.6.1.4.1.318.1.1.1.7.2.6.0)
		upsAdvTestCalibrationResults=${result[$oid_row]#* }
		# int, but defined values are 1 - 3
		is_natural_number "$upsAdvTestCalibrationResults" \
			|| upsAdvTestCalibrationResults=null
		;;
	.1.3.6.1.4.1.318.1.1.1.7.2.7.0)
		upsAdvTestCalibrationDate=${result[$oid_row]#* \"}
		upsAdvTestCalibrationDate=${upsAdvTestCalibrationDate%\"}
		replace_specchars "$upsAdvTestCalibrationDate" \
			&& upsAdvTestCalibrationDate=$without_specchars
		;;
	.1.3.6.1.4.1.318.1.1.1.11.1.1.0)
		upsBasicStateOutputState=${result[$oid_row]#* \"}
		upsBasicStateOutputState=${upsBasicStateOutputState%\"}
		replace_specchars "$upsBasicStateOutputState" \
			&& upsBasicStateOutputState=$without_specchars
		;;
	esac
done

# no values, nothing to do - so exit now
if test $values -eq 0
then
	echo "$(date '+%F %T') $(basename $0) address $address agent $agent snmp_status $snmp_status" >>$errlog
	exit $snmp_status
fi

echo "insert mibapc (agent, time, \
upsBasicIdentModel, \
upsBasicBatteryTimeOnBattery, \
upsAdvBatteryCapacity, \
upsAdvBatteryTemperature, \
upsAdvBatteryRunTimeRemaining, \
upsAdvOutputLoad, \
upsAdvTestDiagnosticsResults, \
upsAdvTestLastDiagnosticsDate, \
upsAdvTestCalibrationResults, \
upsAdvTestCalibrationDate, \
upsBasicStateOutputState \
) values ($agent, unix_timestamp(), \
'$upsBasicIdentModel', \
$upsBasicBatteryTimeOnBattery, \
$upsAdvBatteryCapacity, \
$upsAdvBatteryTemperature, \
$upsAdvBatteryRunTimeRemaining, \
$upsAdvOutputLoad, \
$upsAdvTestDiagnosticsResults, \
'$upsAdvTestLastDiagnosticsDate', \
$upsAdvTestCalibrationResults, \
'$upsAdvTestCalibrationDate', \
'$upsBasicStateOutputState' \
) on duplicate key update \
time = unix_timestamp(), \
upsBasicIdentModel = '$upsBasicIdentModel', \
upsBasicBatteryTimeOnBattery = $upsBasicBatteryTimeOnBattery, \
upsAdvBatteryCapacity = $upsAdvBatteryCapacity, \
upsAdvBatteryTemperature = $upsAdvBatteryTemperature, \
upsAdvBatteryRunTimeRemaining = $upsAdvBatteryRunTimeRemaining, \
upsAdvOutputLoad = $upsAdvOutputLoad, \
upsAdvTestDiagnosticsResults = $upsAdvTestDiagnosticsResults, \
upsAdvTestLastDiagnosticsDate = '$upsAdvTestLastDiagnosticsDate', \
upsAdvTestCalibrationResults = $upsAdvTestCalibrationResults, \
upsAdvTestCalibrationDate = '$upsAdvTestCalibrationDate', \
upsBasicStateOutputState = '$upsBasicStateOutputState' \
;" >&3


rrdfile="$rrddestdir/ups.rrd"

# change NA value from MySQL format (null) to RRD format (U)
test "$upsAdvBatteryRunTimeRemaining" = null && upsAdvBatteryRunTimeRemaining=U

# --start is not necessary
test -f "$rrdfile" || rrdtool create "$rrdfile" --step $rrdstep \
DS:RunTimeRemaining:GAUGE:$rrdheartbeat:0:U \
DS:LineVoltage:GAUGE:$rrdheartbeat:0:U \
RRA:AVERAGE:$rraxff:1:$rrarows \
RRA:AVERAGE:$rraxff:$rrathirtym:$rrarows \
RRA:AVERAGE:$rraxff:$rratwoh:$rrarows \
RRA:AVERAGE:$rraxff:$rraday:$rrarows \
RRA:AVERAGE:$rraxff:$rraweek:$rrarows \
RRA:MIN:$rraxff:1:$rrarows \
RRA:MIN:$rraxff:$rrathirtym:$rrarows \
RRA:MIN:$rraxff:$rratwoh:$rrarows \
RRA:MIN:$rraxff:$rraday:$rrarows \
RRA:MIN:$rraxff:$rraweek:$rrarows \
RRA:MAX:$rraxff:1:$rrarows \
RRA:MAX:$rraxff:$rrathirtym:$rrarows \
RRA:MAX:$rraxff:$rratwoh:$rrarows \
RRA:MAX:$rraxff:$rraday:$rrarows \
RRA:MAX:$rraxff:$rraweek:$rrarows \
RRA:LAST:$rraxff:1:$rrarows \
RRA:LAST:$rraxff:$rrathirtym:$rrarows \
RRA:LAST:$rraxff:$rratwoh:$rrarows \
RRA:LAST:$rraxff:$rraday:$rrarows \
RRA:LAST:$rraxff:$rraweek:$rrarows

rrdtool update "$rrdfile" $rrdcached N:\
$upsAdvBatteryRunTimeRemaining:\
$upsAdvInputLineVoltage

exit $snmp_status
