#!/bin/bash
# Copyright 2016 Michal Žejdl
# snmpget enterprises.9 OID(s) and store to RRD

confdir=$(dirname $0)/../..
config=$confdir/defaults.sh
test -f $config || exit
. $config

functions=$coredir/snmp/functions.sh
test -f $functions || exit
. $functions

address=$1
agent=$2

# a new file descriptor to prevent pipe from closing
exec 3> $mysqlwpipe

# group write permissions for rrdcached
umask 002

rrdstep=$snmpinterval
rrdheartbeat=$((2 * rrdstep))
rraxff=0.5
rrathirtym=$((30 * 60 / rrdstep))
rratwoh=$((4 * rrathirtym))
rraday=$((12 * rratwoh))
rraweek=$((7 * rraday))
rrarows=600

rrddestdir=$rrddir/snmpagent/$agent/Cisco
test -d "$rrddestdir" || mkdir -p "$rrddestdir" || exit

oldifs=$IFS
# one line = one array element; and focus only on the first line
IFS=$'\n'
# no retry (-r 0), do not resend request if OID is missing (-Cf), exit on error
# only Processor MemoryPool (.1)
# ... the processor pool is required to be supported by all devices.
# and cpmCPUTotal5minRev (Rev has values 0..100)
result=($(snmpget -m '' -v 1 -r 0 -c $snmprcommunity -Cf -Onqt $address \
.1.3.6.1.4.1.9.9.48.1.1.1.5.1 \
.1.3.6.1.4.1.9.9.48.1.1.1.6.1 \
.1.3.6.1.4.1.9.9.109.1.1.1.1.8.1 \
))

snmpget_status=$?
# leave if something goes wrong
test $snmpget_status -eq 0 || exit $snmpget_status

IFS=$oldifs

# prepare for inserting into myslq after incomplete response
ciscoMemoryPoolUsed=0
ciscoMemoryPoolFree=0
cpmCPUTotal5minRev=0
# ugly sanitizing
for ((oid_row = 0; oid_row < ${#result[@]}; oid_row++)); do
	case ${result[oid_row]::29} in
	.1.3.6.1.4.1.9.9.48.1.1.1.5.1)
		ciscoMemoryPoolUsed=${result[$oid_row]#.1.3.6.1.4.1.9.9.48.1.1.1.5.1 }
		is_natural_number "$ciscoMemoryPoolUsed" || ciscoMemoryPoolUsed=0
		;;
	.1.3.6.1.4.1.9.9.48.1.1.1.6.1)
		ciscoMemoryPoolFree=${result[$oid_row]#.1.3.6.1.4.1.9.9.48.1.1.1.6.1 }
		is_natural_number "$ciscoMemoryPoolFree" || ciscoMemoryPoolFree=0
		;;
	.1.3.6.1.4.1.9.9.109.1.1.1.1.)
		cpmCPUTotal5minRev=${result[$oid_row]#.1.3.6.1.4.1.9.9.109.1.1.1.1.8.1 }
		is_natural_number "$cpmCPUTotal5minRev" || cpmCPUTotal5minRev=0
		;;
	esac
done

rrdfile="$rrddestdir/MemProc.rrd"

# --start is not necessary
test -f "$rrdfile" || rrdtool create "$rrdfile" --step $rrdstep \
DS:ciscoMemoryPoolUsed:GAUGE:$rrdheartbeat:0:U \
DS:ciscoMemoryPoolFree:GAUGE:$rrdheartbeat:0:U \
DS:cpmCPUTotal5minRev:GAUGE:$rrdheartbeat:0:U \
RRA:AVERAGE:$rraxff:1:$rrarows \
RRA:AVERAGE:$rraxff:$rrathirtym:$rrarows \
RRA:AVERAGE:$rraxff:$rratwoh:$rrarows \
RRA:AVERAGE:$rraxff:$rraday:$rrarows \
RRA:AVERAGE:$rraxff:$rraweek:$rrarows \
RRA:MIN:$rraxff:1:$rrarows \
RRA:MIN:$rraxff:$rrathirtym:$rrarows \
RRA:MIN:$rraxff:$rratwoh:$rrarows \
RRA:MIN:$rraxff:$rraday:$rrarows \
RRA:MIN:$rraxff:$rraweek:$rrarows \
RRA:MAX:$rraxff:1:$rrarows \
RRA:MAX:$rraxff:$rrathirtym:$rrarows \
RRA:MAX:$rraxff:$rratwoh:$rrarows \
RRA:MAX:$rraxff:$rraday:$rrarows \
RRA:MAX:$rraxff:$rraweek:$rrarows \
RRA:LAST:$rraxff:1:$rrarows \
RRA:LAST:$rraxff:$rrathirtym:$rrarows \
RRA:LAST:$rraxff:$rratwoh:$rrarows \
RRA:LAST:$rraxff:$rraday:$rrarows \
RRA:LAST:$rraxff:$rraweek:$rrarows

rrdtool update "$rrdfile" $rrdcached N:\
$ciscoMemoryPoolUsed:\
$ciscoMemoryPoolFree:\
$cpmCPUTotal5minRev
