#!/bin/bash
# Copyright 2010 - 2017 Michal Žejdl
# get ifEntry and ifXEntry, store one part to mysql db and another to rrd

confdir=$(dirname $0)/../..
config=$confdir/defaults.sh
test -f $config || exit
. $config

functions=$coredir/snmp/functions.sh
test -f $functions || exit
. $functions

address=$1
agent=$2
organization=$3
ifXEntry=$4

case "$organization" in
16177)
	# see the note about Westermo switches in agents.sh
	timeout=3
	;;
*)
	# the default according the snmpget man page
	timeout=1
	;;
esac

# a new file descriptor to prevent pipe from closing
exec 3> $mysqlwpipe

# the purpose of one script for ifEntry and ifXEntry
# is to have only one snmpwalk for ifIndex

# no retry (-r 0), do not check for loops (-Cc), values only (-Oqv)
ifIndex=($(snmpwalk -m '' -v 1 -r 0 -t $timeout -c $snmprcommunity -Cc -Oqv \
$address .1.3.6.1.2.1.2.2.1.1 \
))
snmpget_status=$?
# leave if something goes wrong
# nice, but SG500X often returns Timeout: No Response from ... in the middle
#test $snmpget_status -eq 0 || exit
# so go on with whatever snmpwalk returns; the for cycle below do the job
# print an error at least
test $snmpget_status -eq 0 || \
	     echo "$(date +'%F %T') $(basename $0) snmpwalk $address ifIndex \
returned status $snmpget_status; continue with ${#ifIndex[@]} interfaces" >&2

# group write permissions for rrdcached
umask 002

rrdstep=$snmpinterval
rrdheartbeat=$((2 * rrdstep))
rraxff=0.5
rrathirtym=$((30 * 60 / rrdstep))
rratwoh=$((4 * rrathirtym))
rraday=$((12 * rratwoh))
rraweek=$((7 * rraday))
rrarows=600

# ifEntry

rrddestdir=$rrddir/snmpagent/$agent/ifEntry
test -d "$rrddestdir" || mkdir -p "$rrddestdir" || exit

# exclude deprecated ifInNUcastPkts .12, ifOutNUcastPkts .18 and ifOutQLen .21
case "$organization" in
1602)
	# Canon
	ifoids="\
.1.3.6.1.2.1.2.2.1.2 \
.1.3.6.1.2.1.2.2.1.3 \
.1.3.6.1.2.1.2.2.1.4 \
.1.3.6.1.2.1.2.2.1.5 \
.1.3.6.1.2.1.2.2.1.6 \
.1.3.6.1.2.1.2.2.1.7 \
.1.3.6.1.2.1.2.2.1.10 \
.1.3.6.1.2.1.2.2.1.11 \
.1.3.6.1.2.1.2.2.1.13 \
.1.3.6.1.2.1.2.2.1.14 \
.1.3.6.1.2.1.2.2.1.15 \
.1.3.6.1.2.1.2.2.1.16 \
.1.3.6.1.2.1.2.2.1.17 \
.1.3.6.1.2.1.2.2.1.19 \
.1.3.6.1.2.1.2.2.1.20"
	;;
8691)
	# Moxa can handle max 10 oids, so pick up more interesting ones
	ifoids="\
.1.3.6.1.2.1.2.2.1.2 \
.1.3.6.1.2.1.2.2.1.3 \
.1.3.6.1.2.1.2.2.1.4 \
.1.3.6.1.2.1.2.2.1.5 \
.1.3.6.1.2.1.2.2.1.6 \
.1.3.6.1.2.1.2.2.1.10 \
.1.3.6.1.2.1.2.2.1.11 \
.1.3.6.1.2.1.2.2.1.15 \
.1.3.6.1.2.1.2.2.1.16 \
.1.3.6.1.2.1.2.2.1.17"
	;;
*)
	ifoids="\
.1.3.6.1.2.1.2.2.1.2 \
.1.3.6.1.2.1.2.2.1.3 \
.1.3.6.1.2.1.2.2.1.4 \
.1.3.6.1.2.1.2.2.1.5 \
.1.3.6.1.2.1.2.2.1.6 \
.1.3.6.1.2.1.2.2.1.7 \
.1.3.6.1.2.1.2.2.1.8 \
.1.3.6.1.2.1.2.2.1.9 \
.1.3.6.1.2.1.2.2.1.10 \
.1.3.6.1.2.1.2.2.1.11 \
.1.3.6.1.2.1.2.2.1.13 \
.1.3.6.1.2.1.2.2.1.14 \
.1.3.6.1.2.1.2.2.1.15 \
.1.3.6.1.2.1.2.2.1.16 \
.1.3.6.1.2.1.2.2.1.17 \
.1.3.6.1.2.1.2.2.1.19 \
.1.3.6.1.2.1.2.2.1.20"
	;;
esac

for ((i = 0; i < ${#ifIndex[@]}; i++)); do
	index=${ifIndex[i]}
	is_natural_number "$index" || continue

	unset ifoids_index
	member=0
	for ifoid in $ifoids; do
		ifoids_index[member++]=$ifoid.$index
	done
	oldifs=$IFS
	# one line = one array element; and focus only on the first line
	IFS=$'\n'
	# no retry (-r 0), do not resend request if OID is missing (-Cf)
	# -Oa destroys ifPhysAddress, so avoid it
	result=($(snmpget -m '' -v 1 -r 0 -t $timeout \
				-c $snmprcommunity -Cf -Onqt \
				$address ${ifoids_index[@]}))
	snmpget_status=$?
	IFS=$oldifs
	if test $snmpget_status -ne 0; then
		# debugging
		#echo "$(date +'%F %T') $(basename $0) address: $address status: $snmpget_status ifXEntry: $ifXEntry organization: $organization" >&2
		#echo "ifoids_index: ${ifoids_index[@]}" >&2

		# Cisco Catalyst 2960X returns noSuchName on some interfaces
		# so try every interface regardless of previous error
		#break
		continue
	fi

	# prepare for inserting unknown numbers into mysql
	ifType=0
	ifMtu=0
	ifSpeed=0
	ifAdminStatus=0
	ifOperStatus=0
	ifLastChange=0
	ifInOctets=0
	ifInUcastPkts=0
	ifInDiscards=0
	ifInErrors=0
	ifInUnknownProtos=0
	ifOutOctets=0
	ifOutUcastPkts=0
	ifOutDiscards=0
	ifOutErrors=0
	# prepare for inserting unknown strings into mysql
	unset ifDescr ifPhysAddress
	for ((oid_row = 0; oid_row < ${#result[@]}; oid_row++)); do
		case ${result[oid_row]::21} in
		.1.3.6.1.2.1.2.2.1.2.)
			# without MIB (-m '') there are quotes around strings
			ifDescr=${result[oid_row]#.1.3.6.1.2.1.2.2.1.2.$index \"}
			ifDescr=${ifDescr%\"}
			replace_specchars "$ifDescr" && ifDescr=$without_specchars
			;;
		.1.3.6.1.2.1.2.2.1.3.)
			ifType=${result[oid_row]#.1.3.6.1.2.1.2.2.1.3.$index }
			is_natural_number "$ifType" || ifType=0
			;;
		.1.3.6.1.2.1.2.2.1.4.)
			ifMtu=${result[oid_row]#.1.3.6.1.2.1.2.2.1.4.$index }
			is_natural_number "$ifMtu" || ifMtu=0
			;;
		.1.3.6.1.2.1.2.2.1.5.)
			ifSpeed=${result[oid_row]#.1.3.6.1.2.1.2.2.1.5.$index }
			is_natural_number "$ifSpeed" || ifSpeed=0
			;;
		.1.3.6.1.2.1.2.2.1.6.)
			ifPhysAddress=${result[oid_row]#.1.3.6.1.2.1.2.2.1.6.$index \"}
			ifPhysAddress=${ifPhysAddress%\"}
			replace_specchars "$ifPhysAddress" && ifPhysAddress=$without_specchars
			;;
		.1.3.6.1.2.1.2.2.1.7.)
			ifAdminStatus=${result[oid_row]#.1.3.6.1.2.1.2.2.1.7.$index }
			is_natural_number "$ifAdminStatus" || ifAdminStatus=0
			;;
		.1.3.6.1.2.1.2.2.1.8.)
			ifOperStatus=${result[oid_row]#.1.3.6.1.2.1.2.2.1.8.$index }
			is_natural_number "$ifOperStatus" || ifOperStatus=0
			;;
		.1.3.6.1.2.1.2.2.1.9.)
			ifLastChange=${result[oid_row]#.1.3.6.1.2.1.2.2.1.9.$index }
			is_natural_number "$ifLastChange" || ifLastChange=0
			;;
		.1.3.6.1.2.1.2.2.1.10)
			ifInOctets=${result[oid_row]#.1.3.6.1.2.1.2.2.1.10.$index }
			is_natural_number "$ifInOctets" || ifInOctets=0
			;;
		.1.3.6.1.2.1.2.2.1.11)
			ifInUcastPkts=${result[oid_row]#.1.3.6.1.2.1.2.2.1.11.$index }
			is_natural_number "$ifInUcastPkts" || ifInUcastPkts=0
			;;
		.1.3.6.1.2.1.2.2.1.13)
			ifInDiscards=${result[oid_row]#.1.3.6.1.2.1.2.2.1.13.$index }
			is_natural_number "$ifInDiscards" || ifInDiscards=0
			;;
		.1.3.6.1.2.1.2.2.1.14)
			ifInErrors=${result[oid_row]#.1.3.6.1.2.1.2.2.1.14.$index }
			is_natural_number "$ifInErrors" || ifInErrors=0
			;;
		.1.3.6.1.2.1.2.2.1.15)
			ifInUnknownProtos=${result[oid_row]#.1.3.6.1.2.1.2.2.1.15.$index }
			is_natural_number "$ifInUnknownProtos" || ifInUnknownProtos=0
			;;
		.1.3.6.1.2.1.2.2.1.16)
			ifOutOctets=${result[oid_row]#.1.3.6.1.2.1.2.2.1.16.$index }
			is_natural_number "$ifOutOctets" || ifOutOctets=0
			;;
		.1.3.6.1.2.1.2.2.1.17)
			ifOutUcastPkts=${result[oid_row]#.1.3.6.1.2.1.2.2.1.17.$index }
			is_natural_number "$ifOutUcastPkts" || ifOutUcastPkts=0
			;;
		.1.3.6.1.2.1.2.2.1.19)
			ifOutDiscards=${result[oid_row]#.1.3.6.1.2.1.2.2.1.19.$index }
			is_natural_number "$ifOutDiscards" || ifOutDiscards=0
			;;
		.1.3.6.1.2.1.2.2.1.20)
			ifOutErrors=${result[oid_row]#.1.3.6.1.2.1.2.2.1.20.$index }
			is_natural_number "$ifOutErrors" || ifOutErrors=0
			;;
		esac
	done
	echo "insert mibifentry (agent, start, time, \
ifIndex, \
ifDescr, \
ifType, \
ifMtu, \
ifSpeed, \
ifPhysAddress, \
ifAdminStatus, \
ifOperStatus, \
ifLastChange \
) values ($agent, unix_timestamp(), unix_timestamp(), \
$index, \
'$ifDescr', \
$ifType, \
$ifMtu, \
$ifSpeed, \
'$ifPhysAddress', \
$ifAdminStatus, \
$ifOperStatus, \
$ifLastChange \
) on duplicate key update \
time = unix_timestamp(), \
ifDescr = '$ifDescr', \
ifType = $ifType, \
ifMtu = $ifMtu, \
ifSpeed = $ifSpeed, \
ifPhysAddress = '$ifPhysAddress', \
ifAdminStatus = $ifAdminStatus, \
ifOperStatus = $ifOperStatus, \
ifLastChange = $ifLastChange \
	;" >&3

	rrdfile="$rrddestdir/$index.rrd"
	# --start is not necessary
	test -f "$rrdfile" || rrdtool create "$rrdfile" --step $rrdstep \
DS:ifInOctets:COUNTER:$rrdheartbeat:0:U \
DS:ifInUcastPkts:COUNTER:$rrdheartbeat:0:U \
DS:ifInDiscards:COUNTER:$rrdheartbeat:0:U \
DS:ifInErrors:COUNTER:$rrdheartbeat:0:U \
DS:ifInUnknownProtos:COUNTER:$rrdheartbeat:0:U \
DS:ifOutOctets:COUNTER:$rrdheartbeat:0:U \
DS:ifOutUcastPkts:COUNTER:$rrdheartbeat:0:U \
DS:ifOutDiscards:COUNTER:$rrdheartbeat:0:U \
DS:ifOutErrors:COUNTER:$rrdheartbeat:0:U \
RRA:AVERAGE:$rraxff:1:$rrarows \
RRA:AVERAGE:$rraxff:$rrathirtym:$rrarows \
RRA:AVERAGE:$rraxff:$rratwoh:$rrarows \
RRA:AVERAGE:$rraxff:$rraday:$rrarows \
RRA:AVERAGE:$rraxff:$rraweek:$rrarows \
RRA:MIN:$rraxff:1:$rrarows \
RRA:MIN:$rraxff:$rrathirtym:$rrarows \
RRA:MIN:$rraxff:$rratwoh:$rrarows \
RRA:MIN:$rraxff:$rraday:$rrarows \
RRA:MIN:$rraxff:$rraweek:$rrarows \
RRA:MAX:$rraxff:1:$rrarows \
RRA:MAX:$rraxff:$rrathirtym:$rrarows \
RRA:MAX:$rraxff:$rratwoh:$rrarows \
RRA:MAX:$rraxff:$rraday:$rrarows \
RRA:MAX:$rraxff:$rraweek:$rrarows \
RRA:LAST:$rraxff:1:$rrarows \
RRA:LAST:$rraxff:$rrathirtym:$rrarows \
RRA:LAST:$rraxff:$rratwoh:$rrarows \
RRA:LAST:$rraxff:$rraday:$rrarows \
RRA:LAST:$rraxff:$rraweek:$rrarows

	rrdtool update "$rrdfile" $rrdcached N:\
$ifInOctets:\
$ifInUcastPkts:\
$ifInDiscards:\
$ifInErrors:\
$ifInUnknownProtos:\
$ifOutOctets:\
$ifOutUcastPkts:\
$ifOutDiscards:\
$ifOutErrors

done

test "$ifXEntry" = n && exit

# ifXEntry

rrddestdir=$rrddir/snmpagent/$agent/ifXEntry
test -d "$rrddestdir" || mkdir -p "$rrddestdir" || exit

case "$organization" in
311)
	# Microsoft - no .6 .10
	ifxoids="\
.1.3.6.1.2.1.31.1.1.1.1 \
.1.3.6.1.2.1.31.1.1.1.2 \
.1.3.6.1.2.1.31.1.1.1.3 \
.1.3.6.1.2.1.31.1.1.1.4 \
.1.3.6.1.2.1.31.1.1.1.5 \
.1.3.6.1.2.1.31.1.1.1.14 \
.1.3.6.1.2.1.31.1.1.1.15 \
.1.3.6.1.2.1.31.1.1.1.16 \
.1.3.6.1.2.1.31.1.1.1.17"
	;;
8072)
	# Net-SNMP - no .14 and no .17.1
	ifxoids="\
.1.3.6.1.2.1.31.1.1.1.1 \
.1.3.6.1.2.1.31.1.1.1.2 \
.1.3.6.1.2.1.31.1.1.1.3 \
.1.3.6.1.2.1.31.1.1.1.4 \
.1.3.6.1.2.1.31.1.1.1.5 \
.1.3.6.1.2.1.31.1.1.1.6 \
.1.3.6.1.2.1.31.1.1.1.10 \
.1.3.6.1.2.1.31.1.1.1.15 \
.1.3.6.1.2.1.31.1.1.1.16"
	;;
14988)
	# MikroTik
	ifxoids="\
.1.3.6.1.2.1.31.1.1.1.1 \
.1.3.6.1.2.1.31.1.1.1.2 \
.1.3.6.1.2.1.31.1.1.1.3 \
.1.3.6.1.2.1.31.1.1.1.4 \
.1.3.6.1.2.1.31.1.1.1.5 \
.1.3.6.1.2.1.31.1.1.1.6 \
.1.3.6.1.2.1.31.1.1.1.10 \
.1.3.6.1.2.1.31.1.1.1.15"
	;;
*)
	# .7 .11 (ifHCInUcastPkts, ifHCOutUcastPkts) are already collected
	# 	in 32 b ifEntry versions (ifInUcastPkts, ifOutUcastPkts)
	# .8 .9 .12 .13 (ifHCInMulticastPkts, ifHCInBroadcastPkts,
	#	ifHCOutMulticastPkts, ifHCOutBroadcastPkts) are already
	#	collected in 32 b ifXEntry versions (ifInMulticastPkts,
	#	ifInBroadcastPkts, ifOutMulticastPkts, ifOutBroadcastPkts)
	ifxoids="\
.1.3.6.1.2.1.31.1.1.1.1 \
.1.3.6.1.2.1.31.1.1.1.2 \
.1.3.6.1.2.1.31.1.1.1.3 \
.1.3.6.1.2.1.31.1.1.1.4 \
.1.3.6.1.2.1.31.1.1.1.5 \
.1.3.6.1.2.1.31.1.1.1.6 \
.1.3.6.1.2.1.31.1.1.1.10 \
.1.3.6.1.2.1.31.1.1.1.14 \
.1.3.6.1.2.1.31.1.1.1.15 \
.1.3.6.1.2.1.31.1.1.1.16 \
.1.3.6.1.2.1.31.1.1.1.17"
	;;
esac

for ((i = 0; i < ${#ifIndex[@]}; i++)); do
	index=${ifIndex[i]}
	is_natural_number "$index" || continue
	unset ifxoids_index
	member=0
	for ifxoid in $ifxoids; do
		ifxoids_index[member++]=$ifxoid.$index
	done
	oldifs=$IFS
	# one line = one array element; and focus only on the first line
	IFS=$'\n'
	# 2c is min. version for 64b counters
	# no retry (-r 0), do not resend request if OID is missing (-Cf)
	# -Oa destroys ifPhysAddress, so avoid it
	result=($(snmpget -m '' -v 2c -r 0 -t $timeout -c $snmprcommunity \
				-Cf -Onqt $address ${ifxoids_index[@]}))
	snmpget_status=$?
	IFS=$oldifs
	if test $snmpget_status -ne 0; then
		# debugging
		#echo "$(date +'%F %T') $(basename $0) address: $address status: $snmpget_status ifXEntry: $ifXEntry organization: $organization" >&2
		#echo "ifxoids_index: ${ifxoids_index[@]}" >&2

		# Cisco Catalyst 2960X returns noSuchName on some interfaces
		# so try every interface regardless of previous error
		#break
		continue
	fi
	# prepare for inserting unknown numbers into mysql
	ifInMulticastPkts=0
	ifInBroadcastPkts=0
	ifOutMulticastPkts=0
	ifOutBroadcastPkts=0
	ifHCInOctets=0
	ifHCOutOctets=0
	ifLinkUpDownTrapEnable=0
	ifHighSpeed=0
	ifPromiscuousMode=0
	ifConnectorPresent=0
	# prepare for inserting unknown strings into mysql
	unset ifName
	for ((oid_row = 0; oid_row < ${#result[@]}; oid_row++)); do
		case ${result[oid_row]::24} in
		.1.3.6.1.2.1.31.1.1.1.1.)
			# without MIB (-m '') there are quotes around strings
			ifName=${result[oid_row]#.1.3.6.1.2.1.31.1.1.1.1.$index \"}
			ifName=${ifName%\"}
			replace_specchars "$ifName" && ifName=$without_specchars
			;;
		.1.3.6.1.2.1.31.1.1.1.2.)
			ifInMulticastPkts=${result[oid_row]#.1.3.6.1.2.1.31.1.1.1.2.$index }
			is_natural_number "$ifInMulticastPkts" || ifInMulticastPkts=0
			;;
		.1.3.6.1.2.1.31.1.1.1.3.)
			ifInBroadcastPkts=${result[oid_row]#.1.3.6.1.2.1.31.1.1.1.3.$index }
			is_natural_number "$ifInBroadcastPkts" || ifInBroadcastPkts=0
			;;
		.1.3.6.1.2.1.31.1.1.1.4.)
			ifOutMulticastPkts=${result[oid_row]#.1.3.6.1.2.1.31.1.1.1.4.$index }
			is_natural_number "$ifOutMulticastPkts" || ifOutMulticastPkts=0
			;;
		.1.3.6.1.2.1.31.1.1.1.5.)
			ifOutBroadcastPkts=${result[oid_row]#.1.3.6.1.2.1.31.1.1.1.5.$index }
			is_natural_number "$ifOutBroadcastPkts" || ifOutBroadcastPkts=0
			;;
		.1.3.6.1.2.1.31.1.1.1.6.)
			ifHCInOctets=${result[oid_row]#.1.3.6.1.2.1.31.1.1.1.6.$index }
			is_natural_number "$ifHCInOctets" || ifHCInOctets=0
			;;
		.1.3.6.1.2.1.31.1.1.1.10)
			ifHCOutOctets=${result[oid_row]#.1.3.6.1.2.1.31.1.1.1.10.$index }
			is_natural_number "$ifHCOutOctets" || ifHCOutOctets=0
			;;
		.1.3.6.1.2.1.31.1.1.1.14)
			ifLinkUpDownTrapEnable=${result[oid_row]#.1.3.6.1.2.1.31.1.1.1.14.$index }
			is_natural_number "$ifLinkUpDownTrapEnable" || ifLinkUpDownTrapEnable=0
			;;
		.1.3.6.1.2.1.31.1.1.1.15)
			ifHighSpeed=${result[oid_row]#.1.3.6.1.2.1.31.1.1.1.15.$index }
			is_natural_number "$ifHighSpeed" || ifHighSpeed=0
			;;
		.1.3.6.1.2.1.31.1.1.1.16)
			ifPromiscuousMode=${result[oid_row]#.1.3.6.1.2.1.31.1.1.1.16.$index }
			is_natural_number "$ifPromiscuousMode" || ifPromiscuousMode=0
			;;
		.1.3.6.1.2.1.31.1.1.1.17)
			ifConnectorPresent=${result[oid_row]#.1.3.6.1.2.1.31.1.1.1.17.$index }
			is_natural_number "$ifConnectorPresent" || ifConnectorPresent=0
			;;
		esac
	done
	echo "insert mibifxentry (agent, start, time, \
ifIndex, \
ifName, \
ifLinkUpDownTrapEnable, \
ifHighSpeed, \
ifPromiscuousMode, \
ifConnectorPresent \
) values ($agent, unix_timestamp(), unix_timestamp(), \
$index, \
'$ifName', \
$ifLinkUpDownTrapEnable, \
$ifHighSpeed, \
$ifPromiscuousMode, \
$ifConnectorPresent \
) on duplicate key update \
time = unix_timestamp(), \
ifName = '$ifName', \
ifLinkUpDownTrapEnable = $ifLinkUpDownTrapEnable, \
ifHighSpeed = $ifHighSpeed, \
ifPromiscuousMode = $ifPromiscuousMode, \
ifConnectorPresent = $ifConnectorPresent \
	;" >&3

	rrdfile="$rrddestdir/$index.rrd"
	# --start is not necessary
	test -f "$rrdfile" || rrdtool create "$rrdfile" --step $rrdstep \
DS:ifInMulticastPkts:COUNTER:$rrdheartbeat:0:U \
DS:ifInBroadcastPkts:COUNTER:$rrdheartbeat:0:U \
DS:ifOutMulticastPkts:COUNTER:$rrdheartbeat:0:U \
DS:ifOutBroadcastPkts:COUNTER:$rrdheartbeat:0:U \
DS:ifHCInOctets:COUNTER:$rrdheartbeat:0:U \
DS:ifHCOutOctets:COUNTER:$rrdheartbeat:0:U \
RRA:AVERAGE:$rraxff:1:$rrarows \
RRA:AVERAGE:$rraxff:$rrathirtym:$rrarows \
RRA:AVERAGE:$rraxff:$rratwoh:$rrarows \
RRA:AVERAGE:$rraxff:$rraday:$rrarows \
RRA:AVERAGE:$rraxff:$rraweek:$rrarows \
RRA:MIN:$rraxff:1:$rrarows \
RRA:MIN:$rraxff:$rrathirtym:$rrarows \
RRA:MIN:$rraxff:$rratwoh:$rrarows \
RRA:MIN:$rraxff:$rraday:$rrarows \
RRA:MIN:$rraxff:$rraweek:$rrarows \
RRA:MAX:$rraxff:1:$rrarows \
RRA:MAX:$rraxff:$rrathirtym:$rrarows \
RRA:MAX:$rraxff:$rratwoh:$rrarows \
RRA:MAX:$rraxff:$rraday:$rrarows \
RRA:MAX:$rraxff:$rraweek:$rrarows \
RRA:LAST:$rraxff:1:$rrarows \
RRA:LAST:$rraxff:$rrathirtym:$rrarows \
RRA:LAST:$rraxff:$rratwoh:$rrarows \
RRA:LAST:$rraxff:$rraday:$rrarows \
RRA:LAST:$rraxff:$rraweek:$rrarows

	rrdtool update "$rrdfile" $rrdcached N:\
$ifInMulticastPkts:\
$ifInBroadcastPkts:\
$ifOutMulticastPkts:\
$ifOutBroadcastPkts:\
$ifHCInOctets:\
$ifHCOutOctets

done

# pass the exit status of the last snmp* command to the collector.sh
exit $snmpget_status
