#!/bin/bash
# Copyright 2019 Michal Žejdl
# list all STP blocking ports
# but not the ports which are down (ifOperStatus = 2)
# because some switches report down ports as blocking
# (namely 3Com switches with sysObjectID .1.3.6.1.4.1.43.1.16.4.3.*)

# dependencies
for program in mysql
do
	which "$program" >/dev/null || exit
done

confdir=$(dirname $0)/..
config=$confdir/defaults.sh
test -f $config || exit
source $config

mysql -e "\
select sysName, dot1dStpPort port \
from snmpagent \
join dot1dStpPort on id = dot1dStpPort.agent \
join mibportifindex on id = mibportifindex.agent and dot1dStpPort = port \
join mibifentry \
on id = mibifentry.agent and mibportifindex.ifIndex = mibifentry.ifIndex \
where dot1dStpPortState = 2 \
and ifOperStatus != 2 \
and dot1dStpPort.time > unix_timestamp() - 3 * $snmpinterval \
order by sysName, port \
" -h $dbhost -p$dbrpass -u $dbruser $dbname
