#include "confile.h"

// max length of config file line - DNS 255, dbname 64, username 16, password ?
#define BUFFSIZE	512

// malloc(), free()
#include <stdlib.h>
// FILE
#include <stdio.h>
// strtok()
#include <string.h>


struct conf *conf_init(void)
{
	struct conf *conf;
	conf = malloc(sizeof(struct conf));
	if (conf != NULL) {
		conf->dbhost = NULL;
		conf->dbname = NULL;
		conf->dbuser = NULL;
		conf->dbpass = NULL;
		conf->logfilename = NULL;
	}
	return conf;
}

void new_line(FILE *file)
{
	int i;
	while ((i = fgetc(file)) != EOF)
		if (i == '\n') return;
}

void check_pair(struct conf *conf, char *key, char *value)
{
	if ((conf->dbhost == NULL) && (strcmp(key, "dbhost") == 0))
		conf->dbhost = strdup(value);
	if ((conf->dbname == NULL) && (strcmp(key, "dbname") == 0))
		conf->dbname = strdup(value);
	if ((conf->dbuser == NULL) && (strcmp(key, "dbuser") == 0))
		conf->dbuser = strdup(value);
	if ((conf->dbpass == NULL) && (strcmp(key, "dbpass") == 0))
		conf->dbpass = strdup(value);
	if ((conf->logfilename == NULL) && (strcmp(key, "logfilename") == 0))
		conf->logfilename = strdup(value);
}

void conf_print(struct conf *conf)
{
	if (conf->dbhost != NULL)
		printf("dbhost: %s\n", conf->dbhost);
	if (conf->dbname != NULL)
		printf("dbname: %s\n", conf->dbname);
	if (conf->dbuser != NULL)
		printf("dbuser: %s\n", conf->dbuser);
	if (conf->dbpass != NULL)
		printf("dbpass: %s\n", conf->dbpass);
	if (conf->logfilename != NULL)
		printf("logfilename: %s\n", conf->logfilename);
}

void file2conf(FILE *confile, struct conf *conf)
{
	char line[BUFFSIZE], *pos, *left, *right, *key, *value;
	char whitespace[] = " \t\"\'\r\n";
	while (fgets(line, BUFFSIZE, confile) != NULL) {
		// no \n here - line too long or no \n before EOF
		if (strchr(line, '\n') == NULL)
			new_line(confile);
		// cut comments
		if ((pos = strchr(line, '#')) != NULL)
			line[pos - line] = '\0';
		// ignore lines without '='
		if ((pos = strchr(line, '=')) == NULL) continue;
		// ignore lines with more than one '='
		if (strrchr(line, '=') != pos) continue;
		// ^=EOF
		if ((left = strtok(line, "=")) == NULL) continue;
		if ((right = strtok(NULL, "=")) == NULL) continue;
		// no need for next strtok(NULL... - it's only one '=' here
		if ((key = strtok(left, whitespace)) == NULL) continue;
		// one key is enough
		if (strtok(NULL, whitespace) != NULL) continue;
		if ((value = strtok(right, whitespace)) == NULL) continue;
		// one value is enough
		if (strtok(NULL, whitespace) != NULL) continue;
		check_pair(conf, key, value);
	}
	//conf_print(conf);
}

struct conf *conf_get(char *confilename)
{
	FILE *confile;
	struct conf *conf;
	confile = fopen(confilename, "r");
	if (confile == NULL) {
		fprintf(stderr, "Error opening file \"%s\" for reading.\n",
				confilename);
		return NULL;
	}
	conf = conf_init();
	if (conf != NULL) {
		file2conf(confile, conf);
	}
	fclose(confile);
	return conf;
}

void conf_release(struct conf *conf)
{
	if (conf != NULL) {
		free(conf->dbhost);
		free(conf->dbname);
		free(conf->dbuser);
		free(conf->dbpass);
		free(conf->logfilename);
		free(conf);
	}
}
