#!/bin/bash
# Copyright 2010, 2014, 2016 Michal Žejdl
# draw a graph of ifIn/ifOut values from interface RRD file

# at least seven args are required
test $# -lt 7 && exit

rrdbasedir=$1
rrdfile=$2
outputdir=$3
starttime=$4
endtime=$5
ifDescr=$6
selector=$7

# make the remote flush possible
cd $rrdbasedir || exit

test -d "$outputdir" || mkdir -p "$outputdir" || exit

ifIndex=$(basename $rrdfile .rrd)
outputfile=$outputdir/$ifIndex-$starttime-$endtime-$selector.png
# lazy (-z) is handy
options="$outputfile -s $starttime -e $endtime -z"

# append rrdcached
test $# -gt 7 && options="$options --daemon $8"

case $selector in
o)
	label="bits / sec"
	ids=ifInOctets
	ods=ifOutOctets
	;;
u)
	label="ucastPkts / sec"
	ids=ifInUcastPkts
	ods=ifOutUcastPkts
	;;
d)
	label="discards / sec"
	ids=ifInDiscards
	ods=ifOutDiscards
	;;
e)
	label="errors / sec"
	ids=ifInErrors
	ods=ifOutErrors
	;;
n)
	label="unknownProtos / sec"
	ids=ifInUnknownProtos
	;;
m)
	label="mcastPkts / sec"
	ids=ifInMulticastPkts
	ods=ifOutMulticastPkts
	;;
b)
	label="bcastPkts / sec"
	ids=ifInBroadcastPkts
	ods=ifOutBroadcastPkts
	;;
h)
	label="bits / sec"
	ids=ifHCInOctets
	ods=ifHCOutOctets
	;;
*)
	exit
	;;
esac

case $selector in
o|h)
	# octets
	rrdtool graph $options -t "$ifDescr" -v "$label" \
DEF:ioa=$rrdfile:$ids:AVERAGE \
DEF:ion=$rrdfile:$ids:MIN \
DEF:iom=$rrdfile:$ids:MAX \
DEF:iol=$rrdfile:$ids:LAST \
DEF:ooa=$rrdfile:$ods:AVERAGE \
DEF:oon=$rrdfile:$ods:MIN \
DEF:oom=$rrdfile:$ods:MAX \
DEF:ool=$rrdfile:$ods:LAST \
CDEF:iba=ioa,8,* \
CDEF:ibn=ion,8,* \
CDEF:ibm=iom,8,* \
CDEF:ibl=iol,8,* \
CDEF:oba=ooa,8,* \
CDEF:obn=oon,8,* \
CDEF:obm=oom,8,* \
CDEF:obl=ool,8,* \
VDEF:ibaa=iba,AVERAGE \
VDEF:ibnn=ibn,MINIMUM \
VDEF:ibmm=ibm,MAXIMUM \
VDEF:ibll=ibl,LAST \
VDEF:obaa=oba,AVERAGE \
VDEF:obnn=obn,MINIMUM \
VDEF:obmm=obm,MAXIMUM \
VDEF:obll=obl,LAST \
TEXTALIGN:left \
AREA:iba#00CC00:"in " \
GPRINT:ibaa:"avg\: %6.2lf %s" \
GPRINT:ibnn:"min\: %6.2lf %s" \
GPRINT:ibmm:"max\: %6.2lf %s" \
GPRINT:ibll:"lst\: %6.2lf %s\l" \
LINE2:oba#0000CC:"out" \
GPRINT:obaa:"avg\: %6.2lf %s" \
GPRINT:obnn:"min\: %6.2lf %s" \
GPRINT:obmm:"max\: %6.2lf %s" \
GPRINT:obll:"lst\: %6.2lf %s\l"
	;;
n)
	# only In counter
	rrdtool graph $options -t "$ifDescr" -v "$label" \
DEF:ica=$rrdfile:$ids:AVERAGE \
DEF:icn=$rrdfile:$ids:MIN \
DEF:icm=$rrdfile:$ids:MAX \
DEF:icl=$rrdfile:$ids:LAST \
VDEF:icaa=ica,AVERAGE \
VDEF:icnn=icn,MINIMUM \
VDEF:icmm=icm,MAXIMUM \
VDEF:icll=icl,LAST \
TEXTALIGN:left \
AREA:ica#00CC00:"in " \
GPRINT:icaa:"avg\: %6.2lf %s" \
GPRINT:icnn:"min\: %6.2lf %s" \
GPRINT:icmm:"max\: %6.2lf %s" \
GPRINT:icll:"lst\: %6.2lf %s\l" \
COMMENT:" "
	;;
*)
	# counters
	rrdtool graph $options -t "$ifDescr" -v "$label" \
DEF:ica=$rrdfile:$ids:AVERAGE \
DEF:icn=$rrdfile:$ids:MIN \
DEF:icm=$rrdfile:$ids:MAX \
DEF:icl=$rrdfile:$ids:LAST \
DEF:oca=$rrdfile:$ods:AVERAGE \
DEF:ocn=$rrdfile:$ods:MIN \
DEF:ocm=$rrdfile:$ods:MAX \
DEF:ocl=$rrdfile:$ods:LAST \
VDEF:icaa=ica,AVERAGE \
VDEF:icnn=icn,MINIMUM \
VDEF:icmm=icm,MAXIMUM \
VDEF:icll=icl,LAST \
VDEF:ocaa=oca,AVERAGE \
VDEF:ocnn=ocn,MINIMUM \
VDEF:ocmm=ocm,MAXIMUM \
VDEF:ocll=ocl,LAST \
TEXTALIGN:left \
AREA:ica#00CC00:"in " \
GPRINT:icaa:"avg\: %6.2lf %s" \
GPRINT:icnn:"min\: %6.2lf %s" \
GPRINT:icmm:"max\: %6.2lf %s" \
GPRINT:icll:"lst\: %6.2lf %s\l" \
LINE2:oca#0000CC:"out" \
GPRINT:ocaa:"avg\: %6.2lf %s" \
GPRINT:ocnn:"min\: %6.2lf %s" \
GPRINT:ocmm:"max\: %6.2lf %s" \
GPRINT:ocll:"lst\: %6.2lf %s\l"
	;;
esac
