#!/bin/bash
# Copyright 2016 Michal Žejdl
# linePair 1 & 2 lineParameters and performance for CME 20

# at least five args are required
test $# -lt 5 && exit

rrdbasedir=$1
rrdfile=$2
outputdir=$3
starttime=$4
endtime=$5

# make the remote flush possible
cd $rrdbasedir || exit

test -d "$outputdir" || mkdir -p "$outputdir" || exit

# lazy (-z) is handy
options="-s $starttime -e $endtime -z"

# append rrdcached
test $# -gt 5 && options="$options --daemon $6"

filename=$(basename $rrdfile .rrd)

rrdtool graph \
$outputdir/telindus-$filename-lineParameters-$starttime-$endtime.png \
$options -t "lineParameters - lineAttenuation, signalNoise" -v "dB" \
DEF:laa=$rrdfile:lineAttenuation:AVERAGE \
DEF:lan=$rrdfile:lineAttenuation:MIN \
DEF:lam=$rrdfile:lineAttenuation:MAX \
DEF:lal=$rrdfile:lineAttenuation:LAST \
DEF:sna=$rrdfile:signalNoise:AVERAGE \
DEF:snn=$rrdfile:signalNoise:MIN \
DEF:snm=$rrdfile:signalNoise:MAX \
DEF:snl=$rrdfile:signalNoise:LAST \
CDEF:laad=laa,10,/ \
CDEF:land=lan,10,/ \
CDEF:lamd=lam,10,/ \
CDEF:lald=lal,10,/ \
CDEF:snad=sna,10,/ \
CDEF:snnd=snn,10,/ \
CDEF:snmd=snm,10,/ \
CDEF:snld=snl,10,/ \
VDEF:laaa=laad,AVERAGE \
VDEF:lann=land,MINIMUM \
VDEF:lamm=lamd,MAXIMUM \
VDEF:lall=lald,LAST \
VDEF:snaa=snad,AVERAGE \
VDEF:snnn=snnd,MINIMUM \
VDEF:snmm=snmd,MAXIMUM \
VDEF:snll=snld,LAST \
TEXTALIGN:left \
LINE2:laad#CCCC00:"lA" \
GPRINT:laaa:"avg\: %6.2lf %s" \
GPRINT:lann:"min\: %6.2lf %s" \
GPRINT:lamm:"max\: %6.2lf %s" \
GPRINT:lall:"lst\: %6.2lf %s\l" \
LINE2:snad#00CCCC:"sN" \
GPRINT:snaa:"avg\: %6.2lf %s" \
GPRINT:snnn:"min\: %6.2lf %s" \
GPRINT:snmm:"max\: %6.2lf %s" \
GPRINT:snll:"lst\: %6.2lf %s\l"

rrdtool graph \
$outputdir/telindus-$filename-performance-$starttime-$endtime.png \
$options -t "performance - codeViolations" -v "errors" \
DEF:cva=$rrdfile:codeViolations:AVERAGE \
DEF:cvn=$rrdfile:codeViolations:MIN \
DEF:cvm=$rrdfile:codeViolations:MAX \
DEF:cvl=$rrdfile:codeViolations:LAST \
VDEF:cvaa=cva,AVERAGE \
VDEF:cvnn=cvn,MINIMUM \
VDEF:cvmm=cvm,MAXIMUM \
VDEF:cvll=cvl,LAST \
TEXTALIGN:left \
LINE2:cva#CC0000:"cV" \
GPRINT:cvaa:"avg\: %6.2lf %s" \
GPRINT:cvnn:"min\: %6.2lf %s" \
GPRINT:cvmm:"max\: %6.2lf %s" \
GPRINT:cvll:"lst\: %6.2lf %s\l" \
COMMENT:" "
