<?php

/*
 * Copyright 2013 - 2018 Michal Žejdl
 */


// define constants here

define("date_format", "Y&#8209;m&#8209;d H:i:s");

function try_addr2name($ipa) {
	$name = gethostbyaddr($ipa);
	if ($name == $ipa) return "";
	else return "<a href=\"dns.php?id=$name\">$name</a>";
}

function address2name($ipa) {

	$name = gethostbyaddr($ipa);

	if ($name == $ipa)

		return "";
	else
		return $name;
}

function expand_ipv6($address) {
	// convert $address to 16 B and unpack it to the array
	$hexa_array = unpack("H32hexa_string", inet_pton($address));
	// add : after every fourth char and cut the last one :
	return rtrim(preg_replace(
				"/([[:xdigit:]]{4})/", "$1:", $hexa_array["hexa_string"]), ":");
}

function ipv6_compress($address) {

	$compressed = inet_ntop(inet_pton($address));

	if (empty($compressed))

		return $address;
	else
		return $compressed;
}

function debug_print($string) {
	global $debug;
	if ($debug) print("<p>$string\n");
}

function debug_time () {
	global $start_time;
	$delta = round(microtime(true) - $start_time, 3);
	debug_print("time: " . $delta . " second(s)");
}

function sysObjectID2organization ($sysObjectID)
{
	$organizations = array(
			9 => "Cisco",
			43 => "3Com",
			318 => "APC",
			776 => "Telindus",
			12140 => "Alcoma",
			14988 => "MikroTik",
			16177 => "Westermo"
			);

	$numbers = explode(".", $sysObjectID);
	if (array_key_exists(7, $numbers))
		if (array_key_exists($numbers[7], $organizations))
			return $organizations[$numbers[7]];

	return false;
}

function tr_aging_class_test($number)
{
	if ($number % 2)
		$dim = "";
	else
		$dim = "-dim";

	if ($number >= 11)
		return "aging-year".$dim;

	if ($number >= 9)
		return "aging-month".$dim;

	if ($number >= 7)
		return "aging-week".$dim;

	if ($number >= 5)
		return "aging-day".$dim;

	if ($number >= 3)
		return "aging-hour".$dim;

	return "aging-now".$dim;
}

function tr_aging_class($row_number, $delta_time)
{
	if ($row_number % 2)
		$dim = "";
	else
		$dim = "-dim";

	if ($delta_time >= 31536000)
		return "aging-year".$dim;

	if ($delta_time >= 2592000)
		return "aging-month".$dim;

	if ($delta_time >= 604800)
		return "aging-week".$dim;

	if ($delta_time >= 86400)
		return "aging-day".$dim;

	if ($delta_time >= 3600)
		return "aging-hour".$dim;

	return "aging-now".$dim;
}

function date_formated($timestamp)
{
	return date(date_format, $timestamp);
}


// setting locale and domain for gettext(); support for relative paths
function set_i18n_with_document_root($locale, $document_root)
{
	// get it from conf for now; see conf.php-example for the reason
	global $locale;

	// leave now, if the $locale is not set
	if (! isset($locale))
		return;

	$domain = "nana-web";
	$directory = $document_root . "/locale";

	// they said, that this is for MS Windows server
	putenv("LC_MESSAGES=$locale");

	// this is sufficient for me on CentOS Linux
	setlocale(LC_MESSAGES, $locale);

	// the search path for .mo files
	bindtextdomain($domain, $directory);

	/*
	 * gettext() returned question mark instead of national chars
	 * without this function; hardcoded UTF-8 should be changed later
	 */
	bind_textdomain_codeset($domain, "UTF-8");

	// the name of .mo file
	textdomain($domain);
}

function page_open($document_root, $title)
{
	echo <<<EOH
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
<link href="$document_root/style.css" rel="stylesheet" type="text/css">
<title>nana - $title</title>
</head>
<body>
<p>

EOH;

	$html_head="$document_root/head.html";
	if (file_exists($html_head) and is_file($html_head))
		include($html_head);

	require "$document_root/head.php";
}

function page_close($document_root)
{
	global $version;

	echo "<br>\n";

	//print_latest_modifications();

	require "$document_root/tail.php";

	echo <<<EOH

</body>
</html>
EOH;
}


function get_link($link, $name)
{
	return "<a href=\"" . $link . "\"><nobr>" . $name . "</nobr></a>";
}


function print_url($link, $name)
{
	echo get_link($link, $name);
}


function print_url_block($host)
{
	echo "<h2>URL</h2>\n";
	echo "<ul>\n";
	foreach (array("http", "https") as $scheme) {

		echo "<li>";
		$url = $scheme . "://" . $host . "/";
		print_url($url, $url);
		echo "\n";
	}

	echo "</ul>\n";
}


function print_snmp_header($document_root, $snmpagentid)
{
	$result = mysql_query("select
			sysObjectID, sysName,
			dot1dTpFdbEntry, hostResources,	ifXEntry,
			entity
			from snmpagent where id = $snmpagentid");

	if ($row = mysql_fetch_assoc($result)) {

		echo "<h1>SNMP sysName: ", $row["sysName"], "</h1>\n";
		echo "<p>";
		echo "MIB-2: ";
		print_url($document_root . "/snmp/agent.php?id="
				. $snmpagentid,	"system");
		echo ", interfaces - ";
		print_url($document_root . "/snmp/mib/iftable.php?agent="
				. $snmpagentid,	"ifTable");
		echo ", ";
		print_url($document_root . "/snmp/mib/ifgraph.php?agent="
				. $snmpagentid,	gettext("graphs"));

		if ($row["dot1dTpFdbEntry"] == "y") {

			echo ", dot1dBridge ";
			print_url($document_root
					. "/snmp/mib/fdb.php?agent="
					. $snmpagentid,	"FdbTable");
		}

		if ($row["hostResources"] == "y") {

			echo ", ";
			print_url($document_root
					. "/snmp/mib/hostResources.php?agent="
					. $snmpagentid,	"hostResources");
			echo " MIB";
		}

		if ($row["ifXEntry"] == "y") {

			echo ", ifMIB ";
			print_url($document_root
					. "/snmp/mib/ifxtable.php?agent="
					. $snmpagentid,	"ifXTable");
		}

		if ($row["entity"] == "y") {

			echo ", ";
			print_url($document_root
					. "/snmp/mib/entity.php?agent="
					. $snmpagentid,	"entity");
			echo " MIB";
		}

		$dot1dTpFdbEntry = $row["dot1dTpFdbEntry"];

		echo "\n";

		if ($organization = sysObjectID2organization(
					$row["sysObjectID"])) {

			echo "<p>";
			print_url($document_root
					. "/snmp/mib/"
					. $organization . ".php?agent="
					. $snmpagentid,	$organization);

			echo " enterprises MIB";

			switch ($organization) {
				case "APC":
				case "Cisco":
				case "MikroTik":
				case "Westermo":
				case "3Com":
					echo ", ";
					print_url($document_root
							. "/snmp/mib/"
							. $organization
							. ".php",
							sprintf(
								gettext("all %s devices"),
								$organization)
						 );
					break;
			}

			echo "\n";
		}

	} else {
		echo "<p>";
		printf(gettext("SNMP agent no. %s was not found in the database."), $snmpagentid);
		echo "\n";
	}
}


function get_end()
{
	// "now" is for preserving graph image name to use --lazy in rrdgraph

	if (! isset($_GET["end"])) {

		$end = "now";

	} else {

		if ($_GET["end"] == "now") {

			$end = $_GET["end"];

		} else {

			if (! ($end = strtotime($_GET["end"]))) {

				$end = "now";
			}
		}
	}

	return $end;
}


function get_start($end)
{
	/*
	* beware
	* -1d has different meaning for strtotime() than to rrdtool
	* so rather use -1day to simplify case handling
	* strtotime(UNIX timestamp) returns 0 (start of UNIX epoch)
	*/

	// "now-1*" is for preserving graph image name to use --lazy in rrdgraph

	$starts = array("now-1day", "now-1week", "now-1month", "now-1year");

	if (! isset($_GET["start"])) {

		$start = $end . "-1day";

	} else {

		if (in_array($_GET["start"], $starts)) {

			$start = $_GET["start"];

		} else {

			if (! ($start = strtotime($_GET["start"]))) {

				$start = $end . "-1day";

			}
		}
	}

	// prepare UNIX epoch time
	if ($end == "now")
		$end_epoch = strtotime($end);
	else
		$end_epoch = $end;

	if (in_array($start, $starts))
		$start_epoch = strtotime($start);
	else
		$start_epoch = $start;

	// and compare it if $start is older than $end; otherwise modify $start
	if ($start_epoch >= $end_epoch)
		$start = $end . "-1day";

	return $start;
}


function print_start_end_selector($start, $end)
{
	$ranges = array("day", "week", "month", "year");
	$lranges = array("day" => gettext("day"),
			"week" => gettext("week"),
			"month" => gettext("month"),
			"year" => gettext("year"));

	$common_start = false;

	// restore an original query string but without the start and the end
	$query_string = "";
	foreach ($_GET as $key => $value)
		if ($key != "start" and $key != "end")
			$query_string .= $key . "=" . $value . "&amp;";

	$start_link = $_SERVER["SCRIPT_NAME"] . "?" . $query_string
		. "start=now-1";

	echo "<p>\n";

	// $query_string is useless here; the form action arguments are removed
	echo "<form action=\"", $_SERVER["SCRIPT_NAME"], "\">\n";

	echo gettext("show last"), " \n";

	foreach ($ranges as $range) {

		if ($start == "now-1" . $range and $end == "now") {
			$common_start = true;
			echo $lranges[$range];
		} else {
			print_url($start_link . $range, $lranges[$range]);
		}
		if ($range != "year")
			echo ", ";
		echo "\n";
	}

	echo " ";
	echo gettext("or custom range");
	echo " - \n";

	// restore URL query string to hidden input
	foreach ($_GET as $key => $value)
		if ($key != "start" and $key != "end")
			echo "<input type=\"hidden\" name=\"", $key,
			     "\" value=\"", $value, "\">\n";

	echo "start: \n";
	echo "<input type=\"text\" name=\"start\" value=\"";
	if ($common_start)
		echo $start;
	else
		echo $_GET["start"];
	echo "\">\n";
	echo "end: \n";
	echo "<input type=\"text\" name=\"end\" value=\"";
	if ($end == "now")
		echo $end;
	else
		echo $_GET["end"];
	echo "\">\n";
	echo "<input type=\"submit\" value=\"OK\">\n";
	echo "</form>\n";
}


function is_rrd_obsolete($filename, $start)
{
	$starts = array("now-1day", "now-1week", "now-1month", "now-1year");

	if (in_array($start, $starts))
		$start_epoch = strtotime($start);
	else
		$start_epoch = $start;

	if (filemtime($filename) < $start_epoch)
		return true;
	else
		return false;
}


function print_external_link($which, $value)
{
	global $external_link;

	if (!isset($external_link))
		return;

	if (!isset($external_link[$which]["link"])
			or !isset($external_link[$which]["name"]))
		return;

	echo " (";
	print_url(str_replace("%s", $value, $external_link[$which]["link"]),
				str_replace("%s", $value,
					$external_link[$which]["name"]));
	echo "⬈)";
}


function short_hostname($hostname)
{
	$domain = explode(".", $hostname);
	return $domain[0];
}

function print_addr2name_url($ip_address)
{
	$name = gethostbyaddr($ip_address);

	if ($name != $ip_address) {

		print_url("dns.php?id=$name", $name);

		print_external_link("short_hostname", short_hostname($name));
	}
}


function print_external_links($header, $which, $value)
{
	global $external_link;

	$something_printed = false;

	if (!isset($external_link))
		return $something_printed;

	if (!isset($external_link[$which]))
		return $something_printed;

	foreach ($external_link[$which] as $element) {

		if (!isset($element["link"]) or !isset($element["name"]))
			continue;

		if ($header and !$something_printed)
			echo "<h2>", gettext("external links"), "</h2>\n";

		if (!$something_printed) {

			echo "<ul>\n";
			$something_printed = true;
		}

		echo "<li>";
		print_url(str_replace("%s", urlencode($value),
					$element["link"]),
				str_replace("%s", $value, $element["name"]));
		echo "\n";
	}

	if ($something_printed)
		echo "</ul>\n";

	return $something_printed;
}


function get_format_of_coordinates($string)
{
	// 2x degrees minutes seconds
	if (preg_match("/(\d+)[\s°]+(\d+)[\s']+(\d+)(\.\d*)?[\"nNsS,eEwW\s]+(\d+)[\s°]+(\d+)[\s']+(\d+)(\.\d*)?/", $string))
		return 3;

	// 2x degrees minutes
	if (preg_match("/(\d+)[\s°]+(\d+)(\.\d*)?['nNsS,eEwW\s]+(\d+)[\s°]+(\d+)(\.\d*)?/", $string))
		return 2;

	// 2x degrees
	if (preg_match("/(\d+)(\.\d*)?[°nNsS,eEwW\s]+(\d+)(\.\d*)?/", $string))
		return 1;

	return 0;
}

function deg2loc($string)
{
	// .*? is non-greedy anything

	preg_match("/(\d+)(\.\d*)?.*?(\d+)(\.\d*)?/", $string, $matches);

	// $matches[2] can be empty
	$latitude = $matches[1] . $matches[2];
	// but $matches[4] need not to be set
	$longitude = $matches[3] . (isset($matches[4]) ? $matches[4] : "");

	// south
	if (preg_match("/[sS]\s*(\d+)(\.\d*)?.*?(\d+)(\.\d*)?/", $string)
			or preg_match("/(\d+)(\.\d*)?°?\s*[sS].*?(\d+)(\.\d*)?/", $string))
		$latitude = -1 * $latitude;

	// west
	if (preg_match("/(\d+)(\.\d*)?.*?[wW]\s*(\d+)(\.\d*)?/", $string)
			or preg_match("/(\d+)(\.\d*)?.*?(\d+)(\.\d*)?°?\s*[wW]/", $string))
		$longitude = -1 * $longitude;

	return sprintf("%.7f,%.7f", $latitude, $longitude);
}

function degmin2loc($string)
{
	preg_match("/(\d+)[\s°]+(\d+)(\.\d*)?.*?(\d+)[\s°]+(\d+)(\.\d*)?/",
			$string, $matches);

	// $matches[3] can be empty
	$latitude = $matches[1] + ($matches[2] . $matches[3]) / 60;
	// but $matches[6] need not to be set
	$longitude = $matches[4]
		+ ($matches[5] . (isset($matches[6]) ? $matches[6] : "")) / 60;

	// south
	if (preg_match("/[sS]\s*(\d+)[\s°]+(\d+)(\.\d*)?.*?(\d+)[\s°]+(\d+)(\.\d*)?/", $string)
			or preg_match("/(\d+)[\s°]+(\d+)(\.\d*)?'?\s*[sS].*?(\d+)[\s°]+(\d+)(\.\d*)?/", $string))
		$latitude = -1 * $latitude;

	// west
	if (preg_match("/(\d+)[\s°]+(\d+)(\.\d*)?.*?[wW]\s*(\d+)[\s°]+(\d+)(\.\d*)?/", $string)
			or preg_match("/(\d+)[\s°]+(\d+)(\.\d*)?.*?(\d+)[\s°]+(\d+)(\.\d*)?'?\s*[wW]/", $string))
		$longitude = -1 * $longitude;

	return sprintf("%.7f,%.7f", $latitude, $longitude);
}

function degminsec2loc($string)
{
	preg_match("/(\d+)[\s°]+(\d+)[\s']+(\d+)(\.\d*)?.*?(\d+)[\s°]+(\d+)[\s']+(\d+)(\.\d*)?/", $string, $matches);

	/*
	foreach ($matches as $match) {
		echo " x ", $match, " x ";
	}
	*/

	// $matches[4] can be empty
	$latitude = $matches[1] + $matches[2] / 60
		+ ($matches[3] . $matches[4]) / 3600;
	// but $matches[8] need not to be set
	$longitude = $matches[5] + $matches[6] / 60
		+ ($matches[7] . (isset($matches[8]) ? $matches[8] : ""))
		/ 3600;

	// south
	if (preg_match("/[sS]\s*(\d+)[\s°]+(\d+)[\s']+(\d+)(\.\d*)?.*?(\d+)[\s°]+(\d+)[\s']+(\d+)(\.\d*)?/", $string)
			or preg_match("/(\d+)[\s°]+(\d+)[\s']+(\d+)(\.\d*)?\"?\s*[sS].*?(\d+)[\s°]+(\d+)[\s']+(\d+)(\.\d*)?/", $string))
		$latitude = -1 * $latitude;

	// west
	if (preg_match("/(\d+)[\s°]+(\d+)[\s']+(\d+)(\.\d*)?.*?[wW]\s*(\d+)[\s°]+(\d+)[\s']+(\d+)(\.\d*)?/", $string)
			or preg_match("/(\d+)[\s°]+(\d+)[\s']+(\d+)(\.\d*)?.*?(\d+)[\s°]+(\d+)[\s']+(\d+)(\.\d*)?\"?\s*[wW]/", $string))
		$longitude = -1 * $longitude;

	return sprintf("%.7f,%.7f", $latitude, $longitude);
}

function print_external_links_for_location($sysLocation)
{
	global $external_link;

	$format = get_format_of_coordinates($sysLocation);

	switch ($format) {
		case 0:
			return;
		case 1:
			$location = deg2loc($sysLocation);
			break;
		case 2:
			$location = degmin2loc($sysLocation);
			break;
		case 3:
			$location = degminsec2loc($sysLocation);
			break;
	}

	/*
	echo " ", $format;
	echo " location ", $location;
	*/

	if ($location == "")
		return;

	if (!isset($external_link))
		return;

	if (!isset($external_link["location"]))
		return;

	foreach ($external_link["location"] as $element) {
		if (!isset($element["link"]) or !isset($element["name"]))
			continue;

		echo " ";
		print_url(str_replace("%s", urlencode($location),
					$element["link"]),
				str_replace("%s", $location, $element["name"]));
		echo "⬈";
	}
}


function print_info_about_snmp_agent($result)
{
	if ($row = mysql_fetch_array($result)) {

		echo "<h2>SNMP</h2>\n";
		echo "<p>";

		if ($row["status"]) {

			printf(gettext(
						"There was a %sSNMP agent%s on this address. Last successful get: %s."
				      ),
					"<a href=\"snmp/agent.php?id="
					. $row["agent"]
					. "\">",
					"</a>",
					date_formated($row["get"]));

		} else {
			printf(gettext(
						"There is a live %sSNMP agent%s on this address."
				      ),
					"<a href=\"snmp/agent.php?id="
					. $row["agent"]
					. "\">",
					"</a>");
		}

		echo "\n";
	}
}


function print_value_and_unit_or_na($value, $unit)
{
	if (is_null($value)) {

		echo "N/A";

	} else {

		echo "$value";

		if (! is_null($unit)) {

			echo " $unit";
		}
	}
}


function get_link_to_agent($agent_link, $agent, $name)
{
	// for php < 5.5
	$trimmed = trim($name);

	$linked_name = empty($trimmed) ?
		gettext("SNMP agent no.") . " " . $agent : $name;

	return get_link($agent_link . $agent, $linked_name);
}


function print_link_to_agent($agent_link, $agent, $name)
{
	echo get_link_to_agent($agent_link, $agent, $name);
}


function print_link_to_snmp_agent($document_root, $agent, $name)
{
	print_link_to_agent($document_root . "/snmp/agent.php?id=", $agent,
			$name);
}


function print_latest_modifications()
{
	echo "<hr>\n";
	echo "<p>", gettext("latest modifications:"), "\n";
	echo "<ul>\n";
	echo "<li>2018-12-22 - ",
	gettext("mac.php: a name and an address of organization which has registered this MAC prefix"),
	     "\n";
	echo "</ul>\n";
}

?>
